From e18816fee488ae1235460ff2dcbd28cac60f0a3a Mon Sep 17 00:00:00 2001
From: Dirk Mueller <dmueller@suse.com>
Date: Sun, 15 Mar 2015 12:44:37 +0100
Subject: [PATCH 1/2] ARM64: Fix support for ARMv7 userspace

---
 patches.arch/arm64-fix-mmap2-compat-syscall.patch | 73 +++++++++++++++++++++++
 series.conf                                       |  1 +
 2 files changed, 74 insertions(+)
 create mode 100644 patches.arch/arm64-fix-mmap2-compat-syscall.patch

diff --git a/patches.arch/arm64-fix-mmap2-compat-syscall.patch b/patches.arch/arm64-fix-mmap2-compat-syscall.patch
new file mode 100644
index 0000000..0eea377
--- /dev/null
+++ b/patches.arch/arm64-fix-mmap2-compat-syscall.patch
@@ -0,0 +1,73 @@
+From: Andreas Schwab <schwab@suse.de>
+Date: Mon, 9 Mar 2015 17:27:36 +0100
+Subject: [PATCH] arm64: fix implementation of mmap2 compat syscall
+Patch-mainline: Submitted, http://lists.infradead.org/pipermail/linux-arm-kernel/2015-March/330081.html
+References: bnc#923505
+
+The arm mmap2 syscall takes the offset in units of 4K, thus with 64K pages
+the offset needs to be scaled to units of pages.
+
+Tested-by: Alexander Graf <agraf@suse.de>
+Signed-off-by: Andreas Schwab <schwab@suse.de>
+[dmueller: backport to openSUSE 13.2]
+Signed-off-by: Dirk Mueller <dmueller@suse.com>
+
+---
+ arch/arm64/include/asm/unistd32.h |  2 +-
+ arch/arm64/kernel/sys32.S         | 17 +++++++++++++++++
+ 2 files changed, 18 insertions(+), 1 deletion(-)
+
+diff --git a/arch/arm64/include/asm/unistd32.h b/arch/arm64/include/asm/unistd32.h
+index c8d8fc1..5e31f43 100644
+--- a/arch/arm64/include/asm/unistd32.h
++++ b/arch/arm64/include/asm/unistd32.h
+@@ -213,7 +213,7 @@ __SYSCALL(188, sys_ni_syscall)			/* 188 reserved */
+ __SYSCALL(189, sys_ni_syscall)			/* 189 reserved */
+ __SYSCALL(190, sys_vfork)
+ __SYSCALL(191, compat_sys_getrlimit)		/* SuS compliant getrlimit */
+-__SYSCALL(192, sys_mmap_pgoff)
++__SYSCALL(192, compat_sys_mmap2_wrapper)
+ __SYSCALL(193, compat_sys_truncate64_wrapper)
+ __SYSCALL(194, compat_sys_ftruncate64_wrapper)
+ __SYSCALL(195, sys_stat64)
+diff --git a/arch/arm64/kernel/sys32.S b/arch/arm64/kernel/sys32.S
+index 423a5b3..baf024d 100644
+--- a/arch/arm64/kernel/sys32.S
++++ b/arch/arm64/kernel/sys32.S
+@@ -19,9 +19,12 @@
+  */
+ 
+ #include <linux/linkage.h>
++#include <linux/const.h>
+ 
+ #include <asm/assembler.h>
+ #include <asm/asm-offsets.h>
++#include <asm/errno.h>
++#include <asm/page.h>
+ 
+ /*
+  * System call wrappers for the AArch32 compatibility layer.
+@@ -53,6 +56,20 @@ compat_sys_fstatfs64_wrapper:
+ 	b	compat_sys_fstatfs64
+ ENDPROC(compat_sys_fstatfs64_wrapper)
+ 
++
++
++/*
++ * Note: off_4k (w5) is always units of 4K.  If we can't do the requested
++ * offset, we return EINVAL.
++ */
++compat_sys_mmap2_wrapper:
++       tst     w5, #~PAGE_MASK >> 12
++       b.ne    1f
++       lsr     w5, w5, #PAGE_SHIFT - 12
++       b       sys_mmap_pgoff
++1:     mov     x0, #-EINVAL
++       ret     lr
++
+ /*
+  * Wrappers for AArch32 syscalls that either take 64-bit parameters
+  * in registers or that take 32-bit parameters which require sign
+-- 
+2.3.1
+
diff --git a/series.conf b/series.conf
index c371ed0..e0b9f9a 100644
--- a/series.conf
+++ b/series.conf
@@ -234,6 +234,7 @@
 	patches.arch/arm64-0037-drivers-net-fddi-skfp-h-skfbi.h-Remove-useless-PCI_B.patch
 	patches.arch/arm64-0038-arm64-mustang-Disable-sgenet-and-xgenet.patch
 	patches.arch/arm64-0039-generic-pci.patch
+	patches.arch/arm64-fix-mmap2-compat-syscall.patch
 
 	########################################################
 	# S/390
-- 
2.0.4

